package net.knarcraft.permissionsigns.listener;

import net.knarcraft.permissionsigns.PermissionSigns;
import net.knarcraft.permissionsigns.SignManager;
import net.knarcraft.permissionsigns.container.PermissionSign;
import net.knarcraft.permissionsigns.container.SignCreationRequest;
import net.knarcraft.permissionsigns.formatting.TranslatableMessage;
import net.knarcraft.permissionsigns.formatting.Translator;
import org.bukkit.Material;
import org.bukkit.Tag;
import org.bukkit.block.Block;
import org.bukkit.block.Sign;
import org.bukkit.entity.Player;
import org.bukkit.event.EventHandler;
import org.bukkit.event.Listener;
import org.bukkit.event.block.Action;
import org.bukkit.event.player.PlayerInteractEvent;

import java.util.Arrays;

/**
 * A listener for all events related to signs
 */
public class SignListener implements Listener {

    /**
     * This event handler detects if a player clicks a button or a sign
     *
     * @param event <p>The player interact event which was triggered</p>
     */
    @EventHandler
    public void onPlayerInteract(PlayerInteractEvent event) {
        Player player = event.getPlayer();
        Block block = event.getClickedBlock();

        if (block == null) {
            return;
        }

        Material material = block.getBlockData().getMaterial();
        if (!Tag.SIGNS.isTagged(material) && !Tag.WALL_SIGNS.isTagged(material)) {
            return;
        }

        Sign sign = (Sign) block.getState();
        if (event.getAction() == Action.RIGHT_CLICK_BLOCK) {
            handleSignRightClick(sign, player);
        } else if (event.getAction() == Action.LEFT_CLICK_BLOCK) {
            handleSignLeftClick(sign, player, event);
        }
    }

    /**
     * Handles the left click of a sign that might be a permission sign
     *
     * @param sign   <p>The clicked sign</p>
     * @param player <p>The player that clicked the sign</p>
     * @param event  <p>The triggered interaction event</p>
     */
    private void handleSignLeftClick(Sign sign, Player player, PlayerInteractEvent event) {
        boolean registered = SignManager.getSign(sign.getLocation()) != null;
        if (!registered) {
            return;
        }
        if (!player.hasPermission("permissionsigns.admin")) {
            event.setCancelled(true);
            player.sendMessage(Translator.getTranslatedMessage(TranslatableMessage.PERMISSION_SIGN_DESTROY_DENY));
        } else {
            SignManager.removeSign(sign.getLocation());
            player.sendMessage(Translator.getTranslatedMessage(TranslatableMessage.PERMISSION_SIGN_REMOVED));
        }
    }

    /**
     * Handles the right-clicking action
     *
     * @param sign   <p>The clicked sign</p>
     * @param player <p>The player that clicked the sign</p>
     */
    private void handleSignRightClick(Sign sign, Player player) {
        String[] lines = sign.getLines();

        if (SignManager.getSign(sign.getLocation()) != null) {
            //TODO: Perform the permission payment/granting
            return;
        }

        //Don't allow non-empty signs to be overwritten
        if (!Arrays.stream(lines).allMatch(String::isEmpty)) {
            return;
        }

        SignCreationRequest request = PermissionSigns.getSignCreationRequest(player.getUniqueId());
        if (request == null) {
            return;
        }

        //Register the sign and remove the request
        PermissionSign permissionSign = request.getPermissionSign();
        permissionSign.setSignLocation(sign.getLocation());
        SignManager.addSign(permissionSign);
        PermissionSigns.cancelSignCreationRequest(player.getUniqueId());

        //Display the permission sign information on the sign
        String[] signLines = permissionSign.getSignLines();
        for (int i = 0; i < signLines.length; i++) {
            sign.setLine(i, signLines[i]);
        }
        sign.update();
    }

}
